unit Clock;

{
StayOnTop window used to display a frame with H:M:S digits and frame number
shown as a series of 30 tick marks. Every time there is a new frame to display
you have to call ClockBox.ShowFrame to display it. As a side feature, it also
calculates whether the frames are rolling in real time or not, and displays "RT"
if it decides it is real time.

Written by In-key Information Group - www.inkey.com - Feb 1999
}

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, StdCtrls;

type
	FrameString = string[8]; //in form HHMMSSFF

	TClockBox = class(TForm)
		h_label: TLabel;
		m_label: TLabel;
		s_label: TLabel;
		realtime_label: TLabel;
		FramePaintBox: TPaintBox;
		procedure FormShow(Sender: TObject);

	private
		last_h, last_m, last_s, last_f:integer; //the time currently on display
		last_realtime:TDatetime; //the realtime when frame#10 encountered
		last_s_of_f10:integer; //second # when last frame#10 encountered
		Fisrealtime:boolean;

	public
		property IsRealTime:boolean read Fisrealtime;
		procedure ShowFrame(f:FrameString);
	end;

var
	ClockBox: TClockBox;

implementation

{$R *.DFM}

const
	FRAMESPERDAY = 30*60*60*24;

procedure TClockBox.ShowFrame(f:FrameString);
var
	//the current frame time and real time
	cur_h, cur_m, cur_s, cur_f, x:integer;
	cur_realtime, elaps:TDateTime;
begin
	cur_f := StrToIntDef(copy(f, 7, 2), 0);
	if (cur_f = last_f) then exit;
	cur_h := StrToIntDef(copy(f, 1, 2), 0);
	cur_m := StrToIntDef(copy(f, 3, 2), 0);
	cur_s := StrToIntDef(copy(f, 5, 2), 0);

	//decide if clock input does not appear to be running in real time
	if cur_f = 10 then begin //only do once per sec
		cur_realtime := Time;
		if last_s_of_f10 + 1 = cur_s then begin //only do if comparing against a known prior time
			elaps := cur_realtime * FRAMESPERDAY - last_realtime * FRAMESPERDAY;
			if cur_realtime < 2/24/60/60 {the first 2s of the day} then elaps := 30;
				//special case so it doesnt bomb if you are running it over midnight
			Fisrealtime := (elaps > 20) and (elaps < 45);
			realtime_label.visible := Fisrealtime;
		end;
		last_realtime := cur_realtime;
		last_s_of_f10 := cur_s;
	end;

	//update display
	with FramePaintBox do begin
		if cur_f < last_f then begin //reverse colors
			with canvas.pen do if Color <> clBlack then Color := clBlack else Color := clYellow;
		end;
		x := trunc(Width * (cur_f / 30));
		canvas.Rectangle(x, 0, x + 1, 8);
	end;
	last_f := cur_f;

	//only update m-s-h if changed
	if last_s <> cur_s then begin
		last_s := cur_s;
		s_label.caption := copy(f, 5, 2);
		last_m := cur_m;
		m_label.caption := copy(f, 3, 2);
		last_h := cur_h;
		h_label.caption := copy(f, 1, 2);
	end;
end;

procedure TClockBox.FormShow(Sender: TObject);
begin
	Left := Screen.Width - Width;
	Top := 0;
end;

end.
